"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.httpClientPlugin = void 0;
const fastify_plugin_1 = __importDefault(require("fastify-plugin"));
const got_1 = __importDefault(require("got"));
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const utils_1 = require("../utils");
const http_client_instrumentation_1 = require("./http_client_instrumentation");
exports.httpClientPlugin = (0, fastify_plugin_1.default)(initHttpClient, {
    name: "http-client",
    fastify: "3.x",
    decorators: { request: ["correlationId"] },
});
// eslint-disable-next-line @typescript-eslint/require-await
async function initHttpClient(app, opts) {
    const correlationIdHeader = opts.correlationIdHeader || "correlation-id";
    const promMeter = opts.prometheusMeter ||
        (0, utils_1.throwIfUndefined)(app.prometheusMeter, "app.prometheusMeter");
    const metrics = (0, http_client_instrumentation_1.createHttpClientMetrics)(promMeter);
    // Init app-level client
    app.decorate("httpClient", createAppHttpClient(new http_client_instrumentation_1.HttpClientInstrumentation(metrics, app.log), opts.defaultOptions));
    // Init request-specific client
    app.decorateRequest("httpClient", null);
    app.addHook("onRequest", (request, reply, done) => {
        request.httpClient = new HttpClientRequestDecoration(app.httpClient, request.correlationId, correlationIdHeader);
        done();
    });
}
/**
 * Creates an preconfigured HTTP client for the application scope.
 */
function createAppHttpClient(instrumentation, defaultOptions) {
    const alwaysHooks = {
        beforeRequest: [
            (options) => {
                instrumentation.beforeRequest(options);
            },
        ],
        afterResponse: [
            (response) => {
                instrumentation.receivedResponse(response);
                return response;
            },
        ],
        beforeError: [
            (error) => {
                instrumentation.beforeError(error);
                return error;
            },
        ],
        beforeRetry: [
            (options, error, retryCount) => instrumentation.beforeRetry(options, error, retryCount),
        ],
        beforeRedirect: [
            (options, response) => {
                instrumentation.beforeRedirect(options, response);
            },
        ],
    };
    return got_1.default.extend({
        retry: { limit: 0 },
        ...defaultOptions,
        hooks: (0, lodash_merge_1.default)({}, defaultOptions?.hooks, alwaysHooks),
    });
}
class HttpClientRequestDecoration {
    constructor(appHttpClient, correlationId, correlationIdHeader) {
        this.appHttpClient = appHttpClient;
        this.correlationId = correlationId;
        this.correlationIdHeader = correlationIdHeader;
    }
    /**
     * Creates a request-specific HTTP client.
     * @param opts HTTP request options
     */
    create(options) {
        return this.appHttpClient.extend({
            ...options,
            headers: {
                [this.correlationIdHeader]: this.correlationId,
                ...options?.headers,
            },
        });
    }
}
