"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fastify_1 = __importDefault(require("fastify"));
const uuid_1 = require("uuid");
const nock_1 = __importDefault(require("nock"));
const prom_client_1 = __importDefault(require("prom-client"));
const http_client_1 = require("./http_client");
const correlation_id_1 = require("../correlation_id");
const metrics_1 = require("../metrics");
describe("httpClient plugin", () => {
    it("decorates the app with an httpClient", async () => {
        const app = (0, fastify_1.default)()
            .register(correlation_id_1.correlationIdPlugin)
            .register(metrics_1.prometheusMeterPlugin)
            .register(http_client_1.httpClientPlugin);
        await app.ready();
        expect(typeof app.httpClient).toBe("function");
        await app.close();
    });
    it("allows to create a request-specific HTTP client", async () => {
        const app = (0, fastify_1.default)()
            .register(correlation_id_1.correlationIdPlugin)
            .register(metrics_1.prometheusMeterPlugin)
            .register(http_client_1.httpClientPlugin)
            .get("/", (request, reply) => {
            expect(typeof request.httpClient.create).toBe("function");
            expect(typeof request.httpClient.create()).toBe("function");
            void reply.code(200).send();
        });
        await app.ready();
        const res = await app.inject({ method: "GET", url: "/" });
        expect(res.statusCode).toBe(200);
        await app.close();
    });
    it("merges all headers and forwards the correlation-id header", async () => {
        const correlationId = (0, uuid_1.v4)();
        const fakeTargetUrl = "https://test";
        const checkOptionsHook = (opts) => {
            expect(opts.headers).toMatchObject({
                "correlation-id": correlationId,
                "plugin-register-header": "pluginRegister",
                "create-client-header": "createClient",
                "request-header": "request",
            });
        };
        const app = (0, fastify_1.default)()
            .register(correlation_id_1.correlationIdPlugin)
            .register(metrics_1.prometheusMeterPlugin)
            .register(http_client_1.httpClientPlugin, {
            defaultOptions: {
                headers: { "plugin-register-header": "pluginRegister" },
                hooks: { beforeRequest: [checkOptionsHook] },
            },
        })
            .get("/", async (request, reply) => {
            await request.httpClient
                .create({
                headers: { "create-client-header": "createClient" },
            })
                .get(fakeTargetUrl, { headers: { "request-header": "request" } });
            void reply.code(200).send();
        });
        await app.ready();
        const nockScope = (0, nock_1.default)(fakeTargetUrl)
            .get("/")
            .matchHeader("create-client-header", "createClient")
            .matchHeader("request-header", "request")
            .matchHeader("plugin-register-header", "pluginRegister")
            .matchHeader("Correlation-Id", correlationId)
            .reply(200, '{"ok": true}');
        const res = await app.inject({
            method: "GET",
            url: "/",
            headers: { "Correlation-Id": correlationId },
        });
        expect(res.statusCode).toBe(200);
        expect(nockScope.isDone());
        nock_1.default.cleanAll();
        await app.close();
    });
    describe("metrics", () => {
        it("counts HTTPErrors (triggered when configured and status code is not 2xx)", async () => {
            const fakeTargetUrl = "https://testhost";
            const fakeTargetNock = (0, nock_1.default)(fakeTargetUrl)
                .get("/")
                .reply(500, '{"error": "broken"}');
            const prometheusRegistry = new prom_client_1.default.Registry();
            const app = (0, fastify_1.default)()
                .register(correlation_id_1.correlationIdPlugin)
                .register(metrics_1.prometheusMeterPlugin, {
                defaultRegisters: [prometheusRegistry],
            })
                .register(http_client_1.httpClientPlugin)
                .get("/", async (request, reply) => {
                await request.httpClient
                    .create({ throwHttpErrors: true })
                    .get(fakeTargetUrl);
                void reply.code(200).send();
            });
            await app.ready();
            const incErrorCounterSpy = jest.spyOn(prometheusRegistry.getSingleMetric("http_client_request_errors_total"), "inc");
            incErrorCounterSpy.mockClear();
            const res = await app.inject({ method: "GET", url: "/" });
            expect(res.statusCode).toBe(500);
            expect(fakeTargetNock.isDone());
            expect(incErrorCounterSpy).toHaveBeenCalledTimes(1);
            expect(incErrorCounterSpy.mock.calls[0][0]).toStrictEqual({
                host: "testhost",
                error_name: "HTTPError",
                code: 500,
            });
            await app.close();
        });
        it("counts request errors (like ENOTFOUND)", async () => {
            const fakeTargetUrl = "https://testhost";
            const fakeTargetNock = (0, nock_1.default)(fakeTargetUrl)
                .get("/")
                .replyWithError({ code: "ENOTFOUND" });
            const prometheusRegistry = new prom_client_1.default.Registry();
            const app = (0, fastify_1.default)()
                .register(correlation_id_1.correlationIdPlugin)
                .register(metrics_1.prometheusMeterPlugin, {
                defaultRegisters: [prometheusRegistry],
            })
                .register(http_client_1.httpClientPlugin)
                .get("/", async (request, reply) => {
                await request.httpClient
                    .create({ throwHttpErrors: true })
                    .get(fakeTargetUrl);
                void reply.code(200).send();
            });
            await app.ready();
            const incErrorCounterSpy = jest.spyOn(prometheusRegistry.getSingleMetric("http_client_request_errors_total"), "inc");
            incErrorCounterSpy.mockClear();
            const res = await app.inject({ method: "GET", url: "/" });
            expect(res.statusCode).toBe(500);
            expect(fakeTargetNock.isDone());
            expect(incErrorCounterSpy).toHaveBeenCalledTimes(1);
            expect(incErrorCounterSpy.mock.calls[0][0]).toStrictEqual({
                host: "testhost",
                error_name: "RequestError",
                code: "ENOTFOUND",
            });
            await app.close();
        });
    });
});
