"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createHttpClientMetrics = exports.HttpClientInstrumentation = void 0;
const url_1 = require("url");
const got_1 = require("got");
const cached_string_hasher_1 = require("../utils/cached_string_hasher");
class HttpClientInstrumentation {
    constructor(metrics, logger) {
        this.metrics = metrics;
        this.logger = logger;
    }
    beforeRequest(options) {
        this.logger.debug({ http_client_options: createOptionsLog(options) }, "HTTP client starts request");
    }
    receivedResponse(clientResponse) {
        this.logger.debug({ http_client_response: createResponseLog(clientResponse, true) }, "HTTP client has received response");
        if (typeof clientResponse.timings.phases.total === "number") {
            this.metrics.requestDurations.observe({
                host: new url_1.URL(clientResponse.url).host,
                status_code: clientResponse.statusCode,
                is_retry: Number(clientResponse.retryCount > 0),
                from_cache: typeof clientResponse.isFromCache === "boolean"
                    ? Number(clientResponse.isFromCache)
                    : "unknown",
            }, clientResponse.timings.phases.total / 1000);
        }
        else {
            this.logger.error({ http_client_response: createResponseLog(clientResponse) }, "HTTP client request duration metric is missing");
        }
    }
    beforeRetry(options, error, retryCount) {
        const retryLimit = options.retry.limit;
        this.logger.debug({
            retry_count: retryCount,
            retry_limit: retryLimit,
            err: error,
            http_client_options: createOptionsLog(options),
        }, `HTTP client will retry #${retryCount ?? String(retryCount)} of ${retryLimit}`);
    }
    beforeRedirect(options, clientResponse) {
        this.logger.debug({ http_client_response: createResponseLog(clientResponse) }, "HTTP client request will be redirected");
    }
    beforeError(error) {
        const log = { err: error };
        log.http_client_options = createOptionsLog(error.options);
        if (error.response !== undefined) {
            log.http_client_response = createResponseLog(error.response);
        }
        this.metrics.requestErrors.inc({
            host: error.options.url.host || "undefined",
            error_name: error.name,
            code: error instanceof got_1.HTTPError
                ? error.response.statusCode
                : error.code || "undefined",
        });
        this.logger.error(log, `HTTP client error: ${error.message}`);
    }
}
exports.HttpClientInstrumentation = HttpClientInstrumentation;
function createOptionsLog({ url, method, username, password, }) {
    return {
        url: createUrlLog(url),
        method,
        username,
        password: password ? cached_string_hasher_1.cachedStringHasher.hash(password) : undefined,
    };
}
function createResponseLog({ url, requestUrl, statusCode, statusMessage, retryCount, isFromCache, httpVersion, ip, timings, }, includeDurations = false) {
    return {
        requested_url: requestUrl !== url ? createUrlLog(new url_1.URL(requestUrl)) : undefined,
        status_code: statusCode,
        status_message: statusMessage,
        retry_count: retryCount,
        from_cache: isFromCache,
        http_version: httpVersion,
        ip,
        durations: includeDurations ? timings.phases : undefined,
    };
}
function createUrlLog({ protocol, host, pathname, search }) {
    return {
        protocol,
        host,
        path: pathname,
        query_params: search,
    };
}
// Metrics
function createHttpClientMetrics(prometheusMeter) {
    return {
        requestDurations: prometheusMeter.createHistogram({
            name: "http_client_total_request_duration_seconds",
            help: "HTTP client total request durations.",
            labelNames: ["host", "status_code", "is_retry", "from_cache"],
        }),
        requestErrors: prometheusMeter.createCounter({
            name: "http_client_request_errors_total",
            help: "HTTP client total request errors.",
            labelNames: ["host", "error_name", "code"],
        }),
    };
}
exports.createHttpClientMetrics = createHttpClientMetrics;
