"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.requestLoggingPlugin = void 0;
const fastify_plugin_1 = __importDefault(require("fastify-plugin"));
const date_utils_1 = require("../utils/date_utils");
exports.requestLoggingPlugin = (0, fastify_plugin_1.default)(
// eslint-disable-next-line @typescript-eslint/require-await
async function requestLoggingPlugin(app, opts) {
    const requestIdLogLabel = opts.requestIdLogLabel || "request_id";
    app.decorateRequest("receivedAt", null);
    app.addHook("onRequest", (request, reply, done) => {
        // add receivedAt to requests so we can log it on response
        request.receivedAt = new Date();
        // override request and reply logger with a request specific one if it has a child method
        // assert(request.log === reply.log)
        if (typeof request.log.child === "function") {
            request.log = reply.log = request.log.child({
                "correlation-id": request.correlationId,
            });
        }
        done();
    });
    // write access logs
    const accessLogger = app.rootLogger.child({ log_type: "access" });
    app.addHook("onResponse", function (request, reply, done) {
        const { url, queryString } = separateQueryStringFromUrl(request.raw.url);
        const logLevel = typeof reply.context.config.accessLogLevel === "string"
            ? reply.context.config.accessLogLevel
            : "info";
        const logFn = accessLogger[logLevel] || accessLogger.info;
        logFn.call(accessLogger, {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
            [requestIdLogLabel]: request.id,
            msg: `${request.raw.method} ${request.raw.url} ${reply.statusCode}`,
            remote_address: request.ip,
            response_time: Math.round(reply.getResponseTime()),
            received_at: (0, date_utils_1.iso8601WithLocalOffset)(request.receivedAt),
            "correlation-id": request.correlationId,
            request_method: request.raw.method,
            uri: url,
            query_string: queryString,
            status: reply.statusCode,
            user_agent: request.headers["user-agent"] || "",
        });
        done();
    });
}, { decorators: { fastify: ["rootLogger"], request: ["correlationId"] } });
function separateQueryStringFromUrl(url) {
    const querySeparatorIndex = url.indexOf("?");
    if (querySeparatorIndex > -1) {
        return {
            url: url.slice(0, querySeparatorIndex),
            queryString: url.slice(querySeparatorIndex),
        };
    }
    else {
        return { url, queryString: "" };
    }
}
