"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServiceSkeleton = void 0;
const fastify_1 = __importDefault(require("fastify"));
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const fastify_sensible_1 = __importDefault(require("fastify-sensible"));
const under_pressure_1 = __importDefault(require("under-pressure"));
const hyperid_1 = __importDefault(require("hyperid"));
const logging_1 = require("./logging/logging");
const health_check_1 = require("./health_check");
const metrics_1 = require("./metrics");
const http_client_1 = require("./http_client");
const correlation_id_1 = require("./correlation_id");
const request_logging_plugin_1 = require("./logging/request_logging_plugin");
const service_metadata_1 = require("./service_metadata");
const shutdown_1 = require("./shutdown");
const options_1 = require("./utils/options");
function createServiceSkeleton(opts) {
    const { rootLogger, appLogger } = (0, logging_1.createLoggers)(opts.serviceName, opts.logging);
    const finalFastifyOpts = adaptFastifyOptions(appLogger, opts.fastify, opts.logging);
    (0, logging_1.checkLoggingOptionsPlausibility)(appLogger, opts, finalFastifyOpts);
    const app = (0, fastify_1.default)(finalFastifyOpts);
    app.decorate("rootLogger", rootLogger);
    void app.register(service_metadata_1.serviceMetadata, { serviceName: opts.serviceName });
    // Enable all plugins unless explicitly disabled by default,
    // but this behaviour can be modified via `enablePluginsByDefault`
    const isPluginEnabled = (0, options_1.createIsOptionEnabled)(opts.enablePluginsByDefault);
    if (isPluginEnabled(opts.plugins?.healthCheck)) {
        void app.register(health_check_1.healthCheckPlugin, opts.plugins?.healthCheck);
    }
    if (isPluginEnabled(opts.plugins?.correlationId)) {
        void app.register(correlation_id_1.correlationIdPlugin, opts.plugins?.correlationId);
    }
    if (isPluginEnabled(opts.plugins?.requestLogging)) {
        void app.register(request_logging_plugin_1.requestLoggingPlugin, {
            requestIdLogLabel: finalFastifyOpts.requestIdLogLabel,
            ...opts.plugins?.requestLogging,
        });
    }
    if (isPluginEnabled(opts.plugins?.metrics)) {
        void app.register(metrics_1.metricsPlugin, opts.plugins?.metrics);
    }
    if (isPluginEnabled(opts.plugins?.httpClient)) {
        void app.register(http_client_1.httpClientPlugin, opts.plugins?.httpClient);
    }
    if (isPluginEnabled(opts.plugins?.sensible)) {
        void app.register(fastify_sensible_1.default);
    }
    if (isPluginEnabled(opts.plugins?.shutdown)) {
        void app.register(shutdown_1.shutdownPlugin, opts.plugins?.shutdown);
    }
    if (isPluginEnabled(opts.plugins?.underPressure)) {
        void app.register(under_pressure_1.default, opts.plugins?.underPressure);
    }
    return app;
}
exports.createServiceSkeleton = createServiceSkeleton;
function adaptFastifyOptions(appLogger, fastifyOpts, loggingOpts) {
    const defaults = {
        disableRequestLogging: true,
        requestIdLogLabel: "request_id",
        genReqId: (0, hyperid_1.default)({ urlSafe: true }),
    };
    const overrides = {
        logger: (0, options_1.isOptionEnabled)(loggingOpts, true) ? appLogger : false,
    };
    return (0, lodash_merge_1.default)(defaults, fastifyOpts, overrides);
}
